#!/bin/bash

version_dir="lib/.rss"
rss_file="version.rss"
home_page_link="https://rojen.uk/"
rss_link="https://rojen.uk/version.rss"

# Help page
usage() { echo "Usage: ${BASH_SOURCE[0]} [ -v version | -h ]"; }

# Check options
while getopts ":v:h" opt; do
	case $opt in
                v)	version_file="$OPTARG"
			;;
		h)
			usage
			exit 0
			;;
                :)
			echo "Option -$OPTARG requires an argument."
			usage
			exit 1
			;;
		\?)
			echo "Invalid option: -$OPTARG"
			usage
			exit 1
			;;
	esac
done

# Check version
if [[ ! -z $version_file ]]; then
	# Add the version to the file
	if [[ -f "$version_dir/$version_file" ]]; then
		echo "Error: version file $version_file already exists"
		exit 1
	fi
	if [[ ! $version_file =~ ^[0-9]+\.[0-9]+$ ]]; then
		echo "Error: $version_file is not a valid version number"
		exit 1
	fi
	mkdir -p $version_dir
	echo "$(date +'%a, %d %b %Y %T %z')" > $version_dir/$version_file
fi

# Check if version information available
if [[ ! -d $version_dir ]]; then
	echo "No version information available: $(realpath "$version_dir")"
	exit 1
fi

# Assign version information to the array
declare -a versions dates
readarray -t versions < <(ls -1 $version_dir | sort -nr | head -20)
readarray -t dates < <(
	while IFS= read -r get_version_file; do
		cat $version_dir/$get_version_file
	done < <(ls -1 $version_dir | sort -nr | head -20)
)

# Create the RSS XML file
echo '<?xml version="1.0" encoding="UTF-8"?>' > $rss_file
echo '<rss version="2.0" xmlns:atom="http://www.w3.org/2005/Atom">' >> $rss_file
echo '  <channel>' >> $rss_file
echo "    <atom:link href=\"${rss_link}\" rel=\"self\" type=\"application/rss+xml\" />" >> $rss_file
echo '    <title>/rss/ - Rojen Zaman</title>' >> $rss_file
echo '    <link>'$home_page_link'</link>' >> $rss_file
echo '    <description>An RSS feed to track version updates of the rojen.uk</description>' >> $rss_file

for i in "${!versions[@]}"; do
	site_version=${versions[$i]}
	pubDate=${dates[$i]}
	echo '    <item>' >> $rss_file
	echo '      <title>Site version updated to '$site_version'</title>' >> $rss_file
	echo '      <link>'$home_page_link'</link>' >> $rss_file
	echo '      <guid>'$home_page_link#$site_version'</guid>' >> $rss_file
	echo '      <description>The site has been updated to version '$site_version'</description>' >> $rss_file
	echo '      <pubDate>'$pubDate'</pubDate>' >> $rss_file
	echo '    </item>' >> $rss_file
done

echo '  </channel>' >> $rss_file
echo '</rss>' >> $rss_file
